/*******************************************************************************
* FILE NAME: auto_routines.c
*
* DESCRIPTION:
*  This file contains several sample autonomous techniques as a
*  progressive development from primitive to more sophisticated.
*  These use time based dead-reckoning not sensor feedback. The next step
*  beyond these techniques would be PID loops based on gyro or encoders.
*  The different autonomous main programs are named Sample_Auto_n
*  These programs are samples only. There are many ways each of these 
*  can be improved, recombined, modified, etc. to fit particular needs.
*  There are also other ways to accomplish the same ends.
*  More sophisticated isn't necessarily better, because they all accomplish
*  the same thing in the end, and more complexity can make it harder to understand,
*  debug, or modify.
*
*	Sample_Auto_1  -- Basic multi-step autonomous based on the approximate timing of the slow loop
*	Sample_Auto_2  -- One step up from Auto_1, this one does the same thing based on exact timing
*	Sample_Auto_3  -- Creates Functions for standard movements to do the same job as Auto_2
*	Sample_Auto_4  -- A simple time-only scripting approach implementation of Auto_3
*	Sample_Auto_5  -- More complex scripting allows for handling sensor feedback & extra command arguments
* 
*  Scripting could be further complicated by the ability to handle simultaneous operations (drive AND grab),
*     looping within a script, whatever you can imagine and put to use.
*
*******************************************************************************/

#include <stdio.h>
#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "delays.h"       /*defined locally*/
#include "timers.h"

#include "user_routines.h"
#include "auto_routines.h"
#include "clock_routines.h"

#define MAXTIME       254  /* 4+ minutes */
#define RIGHTBALANCE      0 /* used to balance the motors if one is faster than the other */
#define LEFTBALANCE       0
#define FORWARD           1
#define BACKWARD          0

 extern long Right_Encoder_Count;
#define TICKS_PER_INCH	11  // How many encoder ticks equal one inch of robot movement?
// One Vex encoder revolution = 92.5 interrupts =8.75" of travel (just with my particular gearing)

/*******************************************************************************
* FUNCTION NAME: Autonomous_1
* PURPOSE:       Custom autonomous program
*******************************************************************************/
#define MOTORFIX 10

void Autonomous_1(void)
{
  static unsigned int counter=0;
  unsigned int distance_to_travel, distance_traveled;

// Can easily convert to seconds or tenths of seconds simply by dividing by:
//          59 (EDU loops/sec)
//          54 (Vex loops/sec)
//          38 (FRC loops/sec)

// How far should we go?  Simple demo sets desired distance by the placement of a jumper

  if (rc_dig_in03 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 12; // Inches
  else if (rc_dig_in04 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 15; // Inches
  else if (rc_dig_in05 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 18; // Inches
  else if (rc_dig_in06 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 21; // Inches
  else if (rc_dig_in07 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 24; // Inches
  else if (rc_dig_in08 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 27; // Inches
  else if (rc_dig_in09 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 30; // Inches
  else if (rc_dig_in10 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 33; // Inches
  else if (rc_dig_in11 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 36; // Inches
  else if (rc_dig_in12 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 39; // Inches
  else if (rc_dig_in13 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 42; // Inches
  else if (rc_dig_in14 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 45; // Inches
  else if (rc_dig_in15 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 48; // Inches
  else if (rc_dig_in16 == 0)   // When the user's put a jumper on this digital input
	distance_to_travel = 51; // Inches
  else
	distance_to_travel = 6; // Inches

  distance_to_travel *= TICKS_PER_INCH;

  INTCON3bits.INT2IE = 0;  // Stop interrupt from counting (very, very briefly)
	distance_traveled = Right_Encoder_Count;
  INTCON3bits.INT2IE = 1;  // Restart interrupt

    if (counter<54)   // Wait 1 sec. so we can put the robot down after turning it on
	{
		counter++;
		pwm01 = 127;
		pwm02 = 127;
	}
 	else if (distance_traveled < distance_to_travel)   // Drive until we reach the distance asked for
	{
		pwm01 = 254;
		pwm02 = 0 + MOTORFIX;
		printf("Right_Encoder_Count = %ld\r", Right_Encoder_Count);
	}
	else /* Make sure it always stops at the end */
    {
		//printf("Right_Encoder_Count = %ld\r", Right_Encoder_Count);
		pwm01 = 127;
		pwm02 = 127;
	}

} /* End Autonomous_1 */

/*******************************************************************************
* FUNCTION NAME: Sample_Auto_1
* PURPOSE:       Primitive form of autonomous mode program
*                This simply overrides the PWM joystick inputs
* CALLED FROM:   autonomous_mode loop
*******************************************************************************/
void Sample_Auto_1(void)
{
  static unsigned int counter=0;
  unsigned int distance_to_travel, distance_traveled;

 /* Sample primitive autonomous mode */
 /* This uses a counter of the number of slow loops as a measurement for how long 
  * to execute any of the PWM overrides below.
  * For PWMs  0 = full speed in one direction, 127 = stopped or neutral, 
  *     254 = full speed in the opposite direction
  *
  * Can easily convert to seconds or tenths of seconds simply by dividing by:
  *          59 (EDU loops/sec)
  *          54 (Vex loops/sec)
  *          38 (FRC loops/sec)
  
*/

// Delete all printf when needed such


    if (counter<50){
		counter++;
		pwm01 = 127;
		pwm02 = 127;
		//printf ("This is auto 1");
	}
	else if (counter==50){
            INTCON3bits.INT2IE = 0;  // Stop interrupt from counting (very, very briefly)
	            distance_traveled = Right_Encoder_Count;
            INTCON3bits.INT2IE = 1;  // Restart interrupt
            distance_to_travel = (long)12 * TICKS_PER_INCH;
 	         if (distance_traveled < distance_to_travel)   // Drive until we reach this # of inches
	         {
		        pwm01 = 254;
		        pwm02 = 0 + MOTORFIX;
	         }
             else
                counter++; // This will send us to the next timed step
	}
	else if (counter<100){
		counter++;
		pwm01 = 127;
		pwm02 = 127;
		//printf ("Sit Still");
	}
 	else if(counter< 150){
		counter++;
		pwm01 = 0;
		pwm02 = 127;
		//printf ("Going Left");
	}
	else if(counter< 500){
		counter++;
		pwm01 = 0;
		pwm02 = 254;
		//printf ("Going Straight");
	}
	else if(counter<700){
		counter++;
		pwm01=127;
		pwm02=0;
		//printf ("Turning Backward Right");
	}
	else if(counter<900){
		counter++;
		pwm01=254;
		pwm02=0;
		//printf ("Going Backwards");
	}
	else /* Make sure it always stops at the end */
    {
		pwm01 = 127;
		pwm02 = 127;
	}
   

} /* End Sample_Auto_1 */

/*******************************************************************************
* FUNCTION NAME: Sample_Auto_1a
* PURPOSE:       State Machine variation
*******************************************************************************/
#define STATE_1 1
#define STATE_2 2
#define STATE_3 3
#define STATE_4 4
#define STATE_5 5
#define STATE_6 6
#define STATE_7 7

void Sample_Auto_1a(void)
{
  static unsigned int counter=0;
  static char state=STATE_1;

 /* This sometimes uses a counter of the number of slow loops as a measurement for how long 
  * to execute some of the different states.
  * For PWMs  0 = full speed in one direction, 127 = stopped or neutral, 
  *     254 = full speed in the opposite direction
  *
  * Can easily convert to seconds or tenths of seconds simply by dividing by:
  *          59 (EDU loops/sec)
  *          54 (Vex loops/sec)
  *          38 (FRC loops/sec)
*/

	switch (state)
	{
		case STATE_1: 	// Pause for a second 
    			if (counter<50){
				counter++;
				pwm01 = 127;
				pwm02 = 127;
			}
			else{
				counter = 0;
				state = STATE_2;
                printf("STATE_2\r\n");
			}
			break;

		case STATE_2:
			if (counter<30){
				counter++;
				pwm01 = 127;
				pwm02 = 254;
			}
			else{
				counter = 0;
				state = STATE_3;
                printf("STATE_3\r\n");
			}
			break;

		case STATE_3:
 			if(counter< 54){
				counter++;
				pwm01 = 0;
				pwm02 = 127;
			}
			else{
				counter = 0;
				state = STATE_4;
                printf("STATE_4\r\n");
			}
			break;

		case STATE_4:
			if(counter< 100){
				counter++;
				pwm01 = 0;
				pwm02 = 254;
			}
			else{
				counter = 0;
				state = STATE_5;
                printf("STATE_5\r\n");
			}
			break;

		case STATE_5:
			if(counter<40){
				counter++;
				pwm01=127;
				pwm02=0;
			}
			else{
				counter = 0;
				state = STATE_6;
                printf("STATE_6\r\n");
			}
			break;

		case STATE_6:
	 		if(counter<90){
				counter++;
				pwm01=1;
				pwm02=254;
			}
			else{
				counter = 0;
				state = STATE_7;
                printf("STATE_7\r\n");
			}
			break;

		case STATE_7:  // Stop as the last thing you do
			pwm01 = 127;
			pwm02 = 127;
			// Note that state is not changed
			break;

		default:  // Stop if in doubt
			pwm01 = 127;
			pwm02 = 127;
			break;
	}

} /* End Sample_Auto_1a */


/*******************************************************************************
* FUNCTION NAME: Sample_Auto_2
* PURPOSE:       Example of a clock-based autonomous mode.
* CALLED FROM:   autonomous_mode loop
*******************************************************************************/
void Sample_Auto_2(void)
{
  static unsigned int	PreviousCount=254;
  unsigned int stateTime, startCount;
  static char firstTime=TRUE;

  /* Same basic program structure as Sample_1, but uses more reliable time based Clocksec 
   * as the measurement rather than counting program loops
   * The clock must be initialized in User_Initialization() and 
   * "ticks" in InterruptHandlerLow()
  */

   /* pwm01 = LEFT  WHEELS */
   /* pwm02 = RIGHT WHEELS */

  /* An example of sensors you'd check here include limit switches employed as bumpers */

  if (firstTime == TRUE)  // Initialize your counter the very first time through
  {
    firstTime = FALSE;
    startCount = Clocksec;
  }

  if (PreviousCount != Clocksec) /* Example is only once per second 
                                       since we happen to be timing in seconds, but
                                       you'd do this faster (e.g., tenths of seconds or faster)
                                       if you had events that you wanted to occur faster 
                                       For example, if the robot is fast 1 second might be 
                                       10 feet */
  {
     PreviousCount = Clocksec;
     stateTime = Clocksec - startCount;

    if (stateTime < 2){  // If time is less than 2 seconds
		pwm01 = 200;
		pwm02 = 200;

	}
	else if(stateTime < 4){  // If time is less than 4 seconds
		pwm01 = 127;
		pwm02 = 10;

	}
	else if(stateTime < 6){
		pwm01 = 127;
		pwm02 = 127;
	}
	else if(stateTime < 8){
		pwm01=254;
		pwm02=1;

	}
	else if(stateTime < 10){
		pwm01=1;
		pwm02=252;
	}
	else /* Make sure it always stops at the end */
    {
		pwm01 = 127;
		pwm02 = 127;
	}
  }
} /* END Sample_Auto_2 */



/*               ***********************************           */
/**************** Example function-based autonomous ************/
/*               ***********************************           */

/* Function based means you have a collection of basic operations like drive
 * forward/backward, turn-in-place, and more complex like drive until you hit 
 * something then drop your goal hook
*/
/*******************************************************************************
* FUNCTION NAME: Drive_Forward
* PURPOSE:       Simple drive straight function for function-based autonomous mode.
* CALLED FROM:   file:user_routines_fast.c, Sample_Auto_3
* ARGUMENTS:     input: speed 0-127
*                output: pwm01,pwm02
* p1=left, p2=right
* LEFT & RIGHTBALANCE are used to account for one motor being more powerful 
* or one drive train being more efficient than the other. It assumes a simple
* balance constant proportional to the speed of the robot (speed/fullspeed).
* This example also assumes the motors are mounted so that one must be driven in reverse
* while the other is driven forward.
*
*******************************************************************************/
void Drive_Forward(unsigned char speed)
{

    pwm01 = 127 - speed - (RIGHTBALANCE*speed/127);
    pwm02 = 127 + speed + (LEFTBALANCE*speed/127); // Assumes L/R motors are opposite
}

/*******************************************************************************
* FUNCTION NAME: Drive_Backwards
* PURPOSE:       Function-based autonomous mode.
* CALLED FROM:   file:user_routines_fast.c, Sample_Auto_3
* ARGUMENTS:     input: speed 0-127
*                output: pwm01,pwm02 
* RETURNS:       void
*******************************************************************************/
void Drive_Backwards(unsigned char speed)
{
  /* p1=left, p2=right */

    pwm01 = 127 + speed + (RIGHTBALANCE*speed/127);
    pwm02 = 127 - speed - (LEFTBALANCE*speed/127);
}

/*******************************************************************************
* FUNCTION NAME: Drive_Left
* PURPOSE:       Function-based autonomous mode.
* CALLED FROM:   file:user_routines_fast.c, Sample_Auto_3
* ARGUMENTS:     input: speed 0-127
*                output: pwm01,pwm02
* RETURNS:       void
*******************************************************************************/
void Drive_Left(unsigned char speed, unsigned char direction)
{
  /* p1=left, p2=right */
  if (direction == FORWARD)
  {
    pwm01 = 127 - speed;
    pwm02 = 127;
  }else /* backup turn */
  {
    pwm01 = 127;
    pwm02 = 127 - speed;
  }
}

/*******************************************************************************
* FUNCTION NAME: Drive_Right
* PURPOSE:       Function-based autonomous mode.
* CALLED FROM:   file:user_routines_fast.c, Sample_Auto_3
* ARGUMENTS:     input: speed 0-127
*                output: pwm01,pwm02
* RETURNS:       void
*******************************************************************************/
void Drive_Right(unsigned char speed, unsigned char direction)
{
  /* p1=left, p2=right */
  
  if (direction == FORWARD)
  {
    pwm01 = 127;
    pwm02 = 127 + speed;
  }else /* backup turn */
  {
    pwm01 = 127 + speed;
    pwm02 = 127;
  }
}

/*******************************************************************************
* FUNCTION NAME: Drive_Spin_In_Place
* PURPOSE:       Function-based autonomous mode.
* CALLED FROM:   file:user_routines_fast.c, Sample_Auto_3
* ARGUMENTS:     input: speed 0-127
*                output: pwm01,pwm02
* RETURNS:       void
*******************************************************************************/
void Drive_Spin_In_Place(unsigned char speed, unsigned char direction)
{
  /* p1=left, p2=right */
  
  if (direction == CLOCKWISE) /* Spin Clockwise */
  {
    pwm01 = 127 - speed;
    pwm02 = 127 - speed;

  }else /* Spin the opposite direction - Counter Clockwise */
  {
    pwm01 = 127 + speed;
    pwm02 = 127 + speed;
  }
}

/*******************************************************************************
* FUNCTION NAME: Drive_Stop
* PURPOSE:       Function-based autonomous mode.
* CALLED FROM:   file:user_routines_fast.c, Sample_Auto_3
* ARGUMENTS:     input: speed 0-127
*                output: pwm01,pwm02
* RETURNS:       void
*******************************************************************************/
void Drive_Stop(void)
{
  pwm01 = NEUTRAL;
  pwm02 = NEUTRAL;
}

/*******************************************************************************
* FUNCTION NAME: Sample_Auto_3
* PURPOSE:       Directly program the autonomous logic
*    pwm01 = RIGHT WHEELS 
*    pwm02 = LEFT WHEELS 
*******************************************************************************/
#define TURNTIME 10  // in tenths of seconds, e.g., 10 = 1 second
#define TIMETOSTOP 30 // in seconds
void Sample_Auto_3(void)
{
  static unsigned char	State=0; //Two States 0=drive until collision, 1=backup turn
  static unsigned char  stopTime=MAXTIME, firstTime=TRUE, turnTime=0;


  if (firstTime)   /* allows for auto starting at any time */
  {
     firstTime = FALSE;
     stopTime = Clocksec + TIMETOSTOP; // Want to stop after 60 seconds
  }
/*******************/
/* Check for walls */

  if (!rc_dig_in16)
  {
    State = 1; // Collision, so backup and turn
    turnTime = Clockds + TURNTIME; // Decide how long to turn in tenths of seconds
  }
  else if (turnTime < Clockds)
  {
    State = 0; // Cease backing up and turning
  }
/***********************************/
/* Which way should we be driving? */

  if (State == 1)
  {
    Drive_Left(HALFSPEED, BACKWARD); //speed = 1-127
  }
  else
  {
    Drive_Forward (HALFSPEED); //speed = 1-127
  }
/************************************/
/* Stop driving around after awhile */

  if (stopTime < Clocksec)  // Need some way to stop after awhile
  {
    Drive_Stop();
  }
  
} /* END Sample_Auto_3 */



/*******************************************************************************
* FUNCTION NAME: Sample_Auto_4
* PURPOSE:       Example of a script-based autonomous mode.
*                One of the nice benefits of scripted commands is the ease with which
*                you can develop a program to graphically generate the script. e.g.,
*                a VB program that displays a picture of the playing field the user
*                can draw the path of the robot on, then the program can print out
*                the list of commands to be pasted into this code.
* CALLED FROM:   file: user_routines_fast.c, User_Autonomous
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Sample_Auto_4(void)
{
 /* Same basic program idea as Sample_Auto_3, but collects the driving 
  * commands in a single array
 */

   /* Driving commands */
  #define ENDCMD  0  /* Must always be the last command */
  #define DF            	1  /* Drive Forward */
  #define DB            	2  /* Drive Backward */
  #define DL            	3  /* Drive Left */
  #define DR            	4  /* Drive Right */
  #define DS           	5  /* Spin in place */
  #define DSTOP      6  /* Stop all engines */
  #define DLOOP      7  /* Jump to cmd */

  /* command array indexes */
  #define CMDTIME       0
  #define CMD           1
  #define CMDSPEED      2
  #define CMDDIRECTION  3

  static unsigned char command, time=0, speed, direction;
  static unsigned char cmdNum=0, startTime=MAXTIME;
  static unsigned char firstTime=TRUE;


  /*********************************************/
  /*********************************************/
  /*** This is the autonomous command script ***/

  static rom unsigned char  CommandScript[20][4] = 
  { /* time(do cmd for this # of seconds), command, speed, direction */
    {4,  DSTOP, FULLSPEED,  	NONE    }, 
    {6,  DF,    	FULLSPEED,  	NONE    },
    {8,  DL,   	 HALFSPEED,  BACKWARD},
    {10, DS,    	FULLSPEED,  	BACKWARD},
    {11, DB,    	HALFSPEED,  	NONE    },
    {12, DR,    	FULLSPEED,  	BACKWARD},
    {13, DSTOP, NONE,       		NONE    },
    {14, ENDCMD,NONE,      	NONE    }
  };

  /******** End autonomous script ********/
  /*****************************************/
  /*****************************************/

  if (firstTime)   /* allows for auto starting at any time */
  {  
     firstTime = FALSE;
     startTime = Clocksec; /* record when autonomous begins */
  }

  if (time <= Clocksec) /* Check to see when time on the previous command has run out */
  {
    /* Go on to the next command */
     command   = CommandScript[cmdNum][CMD];
     time      = startTime + CommandScript[cmdNum][CMDTIME];
     speed     = CommandScript[cmdNum][CMDSPEED];
     direction = CommandScript[cmdNum][CMDDIRECTION];
 
    if (command != ENDCMD)
       cmdNum++; /* Ready for the command after this */

    switch (command)
    {
      case DF:
          Drive_Forward(speed);
          break;
      case DL:
          Drive_Left(speed, direction);
          break;
      case DS:
          Drive_Spin_In_Place(speed, direction);
          break;
      case DB:
          Drive_Backwards (speed);
          break;
      case DR:
          Drive_Right(speed, direction);
          break;
      case DSTOP:
          Drive_Stop();
          break;
      case ENDCMD:
      default:
          time = MAXTIME;
          Drive_Stop();
    } /* End switch */
  } /* End elapsed time check */
} /* END Sample_Auto_4 */



/*******************************************************************************
* FUNCTION NAME: Sample_Auto_5
* PURPOSE:       Example of a sequential script-based event-driven autonomous mode.
*                In Sample_Auto_4 time was the only type of event. This expands 
*                that to include a digital switch and allows for many types of events.
*                Do this until the event happens, then go on to the next event/cmd.
*                There are lots of variants and changes that can be made to this
*                basic design concept.
*                This can be expanded to respond to unexpected events too, like 
*                obstacle avoidance.
*                Remember these sample drive commands use time, but distance,
*                coordinates, or other variations can also be used.
*                Space can also be saved by defining CommandScript as a 1D array
*                and context-based variable length events/commands.
* CALLED FROM:   file: user_routines_fast.c, User_Autonomous
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Sample_Auto_5(void)
{
 /* Builds on Sample_Auto_4, but substitutes multiple types of events for the 
  * single event of time.
 */

   /* Driving commands */
//  #define ENDCMD       0  /* Must always be the last command */
//  #define DF           		1  /* Drive Forward */
//  #define DB           	2  /* Drive Backward */
//  #define DL            	3  /* Drive Left */
//  #define DR            	4  /* Drive Right */
//  #define DS            	5  /* Spin in place */
//  #define DLOOP        	7  /* Jump to cmd */
//  #define DSTOP         	6  /* Stop all engines */
  /* If a command does not use the extra parameters allowed then enter "NONE" */

  /* Event types */
  #define ETIME         1  /* Time event */
  #define EDIGITAL      2  /* Switch event, e.g., limit or toggle switch on */
  #define EANALOG       3  /* Analog type event like a pot reaching a value */
  #define EINTERRUPT    4  /* Other interrupt driven event */

  /*** Event Descriptions ***/
  /** The command is executed until the event occurs */
  /** If you want to execute the command after the event occurs then put the */
  /** command in the next script line */
  /* "ETIME time" - will perform the command for a duration of "time" */
  /* "EDIGITAL input" - will wait until "input" is ON */
  /* "EANALOG input value" - will wait until analog "input" has the specified value */
  /* "EINTERRUPT input" - will wait until an interrupt from "input" is received */
  /* "NONE" - perform command immediately */


  /* command array indexes */
  #define CMD5EVENT      0
  #define CMD5EVENTP1    1
  #define CMD5           2
  #define CMD5SPEED      3
  #define CMD5DIRECTION  4

  #define MAXCMD 254

  static unsigned char event, eventP1, command, speed, direction;
  static unsigned char cmdNum=MAXCMD, startTime, nextCmd=TRUE, eventFlag=FALSE;
  static unsigned char newCmd;


  /*********************************************/
  /*** This is the autonomous command script ***/
  /* 'rom' isn't necessary, but puts the variable into the larger 
   * program space thereby preserving the more limited data space */

  static rom unsigned char  CommandScript[20][5] = 
  { /* event & parameters (like time), command & parameters (like speed, direction) */
    {ETIME,    		2,    DSTOP,  NONE,       	NONE    }, /* Give ourselves time to put it down */
    {EDIGITAL, 		1,    DF,     HALFSPEED,  	FORWARD }, /* wait for switch 16 to be on */
    {ETIME,    		1,    DR,     FULLSPEED,  	BACKWARD},
    {NONE,    		1,    DLOOP,     1,  			NONE},
    {ENDCMD,  	NONE,  NONE,   	NONE,       	NONE    }
  }; 

  /******* End autonomous command script ********/
  /**********************************************/

  /* Check for all events that have occurred */
  /* Actual event processing should be performed in user_routines_fast.c */
  
  /* Ready to load the next command */
  if (nextCmd)
  {
     nextCmd = FALSE;
     if (cmdNum == MAXCMD)
       cmdNum = 0;
     else
       cmdNum++;
     event     = CommandScript[cmdNum][CMD5EVENT];
     eventP1   = CommandScript[cmdNum][CMD5EVENTP1];
     eventFlag = TRUE; /* Execute the event command */
     newCmd    = TRUE;
  }

  /** Execute the event command **/
  if (eventFlag) 
  {
    eventFlag = FALSE; /* Don't bother repeating the command each loop */
    /* Read the command */
    command   = CommandScript[cmdNum][CMD5];
    speed     = CommandScript[cmdNum][CMD5SPEED];
    direction = CommandScript[cmdNum][CMD5DIRECTION];

    switch (command)
    {
      case DF:
          Drive_Forward(speed);
          break;
      case DL:
          Drive_Left(speed, direction);
          break;
      case DS:
          Drive_Spin_In_Place(speed, direction);
          break;
      case DB:
          Drive_Backwards (speed);
          break;
      case DR:
          Drive_Right(speed, direction);
          break;
      case DSTOP:
          Drive_Stop();
          break;
      case DLOOP:
          if(speed < 1) speed = 1; // Cannot branch to cmd 0
          cmdNum = speed-1;  // "speed" is really the command (1-n) to branch (back) to
          break;
      case ENDCMD:
      default:
          Drive_Stop();
    } /* End command switch */
  } /* End eventFlag check */

  switch (event)
  { /* Until the event occurs the command is executed */
    case ETIME:
      if (newCmd)
      { /* Need this flag to find the clock time when the event will be done */
        eventP1+=Clocksec;
        newCmd = FALSE;
      }
      if (eventP1 < Clocksec)
      { /* Time on the previous command has run out */
        nextCmd = TRUE;
      }
      break;
    case EDIGITAL:
      switch (eventP1)
      {
        case 1:
          nextCmd = !rc_dig_in16; /* Use a switch on digital input 1 */
                                  /* Connect switch to pins BLK & SIG */
          break;
//        case 2:
//          nextCmd = !rc_dig_in10; /* Use a switch on digital input 2 */
//          break;
//        case 3:
//          nextCmd = !rc_dig_in12; /* Use a switch on digital input 3 */
//          break;
        default:
          nextCmd = TRUE;
      }
      break;
    case NONE:
      nextCmd = TRUE;
      break;
    case ENDCMD:
      nextCmd = FALSE;
      break;
    default:
      nextCmd = FALSE;

  } /* End event switch */

} /* END Sample_Auto_5 */
